<!---
			MultipartMime
			
			Component to construct Multipart MIME
			messages.
			--->
<cfcomponent>


	<!---
				CFC's store private component level data in
				the variables scope.  However, there is a
				bug in CFMX where variables prefixing causes
				data to be confused with the calling template.
				
				To work around this limitation, we create a
				variables scope structure to hold all other
				private data variables.
				--->
	<cfset private=StructNew()>


	<!---
				Set up component level variables for the
				message parts, the header, and footer.
				--->
	<cfset CRLF=Chr(13) & Chr(10)>
	<cfset private.msgParts=ArrayNew(1)>
	<cfset private.header="This document is a Multipart MIME message.  " & 
											  "If you are seeing this text then the message " & 
												"was not rendered properly.#CRLF##CRLF#" & 
												"If you are an end-user, contat the message " & 
												"originator and inform them of the problem.#CRLF##CRLF#" & 
												"If you are a developer, refer to RFCs 2046 for " & 
												"more information.">
	<cfset private.footer="">
	
	
	<!---
				getRandText
				
				utility function for creating random text.
				Used in creating the boundary and is useful
				in creating content locations.  Returns a 
				random hex string.
				--->
	<cffunction
		name="getRandText"
		returnType="string"
		access="public"
		output="false">
		
		<cfset var r= FormatBaseN(RandRange(1000000,900000),16) & FormatBaseN(RandRange(1000000,900000),16)>
		
		<cfreturn r>
		
	</cffunction>


	<!---
				addPart
				
				Adds a message part to the Multipart MIME 
				message.  Accepts three arguments.
				
				content		- the content of the message part.
										Must be either a string or binary.
				location	-	The text for the content-location
										MIME header field.
				type			-	The text for the content-type
										MIME header field.
				headers		-	Any additional MIME headers, in
										key/value pairs.  The keys will be
										the MIME headers and values copied
										as is to the message part header.
				--->
	<cffunction
		name="addPart"
		returnType="void"
		access="public"
		output="false">
		
		<cfargument
			name="content"
			type="any"
			required="true" />
		
		<cfargument
			name="location"
			type="string"
			required="false" />
		
		<cfargument
			name="type"
			type="string"
			required="false" />
			
		<cfargument
			name="headers"
			type="struct"
			required="false" />
		
		<cfset var thisPart=StructNew()>
		
		
		<!---
					Validate that content is either a string
					or binary.
					--->
		<cfif Not (isSimpleValue(arguments.content) OR
							 isBinary(arguments.content))>
			<cfthrow 
				type="application"					 
				message="MultipartMIME.addPart: Content must be a string or binary." />
			
		</cfif>
		
		<cfset thisPart.content=arguments.content>
		<cfset thisPart.headers=StructNew()>
		
		<cfif structKeyExists(arguments,"location")>
			<cfset structInsert(thisPart.headers,"Content-Location",arguments.location)>	
		</cfif>
		
		<cfif structKeyExists(arguments,"type")>
			<cfset structInsert(thisPart.headers,"Content-Type",arguments.type)>
		</cfif>
		
		<cfif structKeyExists(arguments,"headers")>
			<cfset structAppend(thisPart.headers,arguments.headers,"Yes")>
		</cfif>
		
		<cfset arrayAppend(private.msgParts,thisPart)>
		
	</cffunction>
	
	
	<!---
				setHeader
				
				Sets the text header for the message.
				--->
	<cffunction	
		name="setHeader"
		returnType="void"
		access="public"
		output="false">
		
		<cfargument
			name="content"
			type="string"
			required="true" />
			
		<cfset private.header=arguments.content>
			
	</cffunction>


	<!---
				setFooter
				
				Sets the text footer for the message.
				--->
	<cffunction	
		name="setFooter"
		returnType="void"
		access="public"
		output="false">
		
		<cfargument
			name="content"
			type="string"
			required="true" />
			
		<cfset private.footer=arguments.content>
			
	</cffunction>


	<!---
				getMessage
				
				Puts together all the parts and return the
				Multipart MIME message.
				--->
	<cffunction
		name="getMessage"
		returnType="string"
		access="public"
		output="false">
		
		<cfset var i=0>
		<cfset var k="">
		<cfset var contentIsBinary="">
		

		<!---
					A Multipart MIME message part can actually 
					contain another Multipart MIME message as 
					it's content.  Therefore the boundary must
					be a very unique string.  For that reason
					we use a utiltiy function in this component
					to put together the boundary.
					--->
		<cfset var boundary="----=_" & getRandText()>
		
		
		<!---
					The message starts with the MIME headers, which
					is just the version, content type, and boundary.
					--->
		<cfset var msg="MIME-Version: 1.0#CRLF#" & 
									 "Content-Type: multipart/related; boundary=""#boundary#""">

		
		<!---
					Add the message header
					--->
		<cfset msg=msg & CRLF & CRLF & private.header & CRLF & CRLF>
		

		<!---
					Validate that we have at least one message part.
					--->
		<cfif NOT ArrayLen(private.msgParts)>
			
			<cfthrow 
				type="application"
				message="No message parts were added to MultipartMIME message."/>
				
		</cfif>


		<!---
					loop through the parts and add each one
					--->
		<cfloop index="i" from="1" to="#ArrayLen(private.msgParts)#">
			
			
			<!---
						Each part starts with two dashes and the boundary
						--->
			<cfset msg=msg & "--" & boundary & CRLF>
			
			
			<!---
						Add the headers for this part.
						--->
			<cfloop item="k" collection="#private.msgParts[i].headers#">
				<cfset msg=msg & k & ": " & private.msgParts[i].headers[k] & CRLF>
			</cfloop>
			
			
			<!---
						Add encoding type header and indicate end
						of headers.
						--->
			<cfset msg=msg & "Content-Transfer-Encoding: ">
			<cfset contentIsBinary=isBinary(private.msgParts[i].content)>
			<cfif contentIsBinary>
				<cfset msg=msg & "base64">
			<cfelse>
				<cfset msg=msg & "quoted-printable">
			</cfif>			
			<cfset msg=msg & CRLF & CRLF>			
			
			
			<!---
						Add the part content, after appropriate encoding.
						--->
			<cfif contentIsBinary>
				<cfset msg=msg & toBase64Short(private.msgParts[i].content) & CRLF>
			<cfelse>
				<cfset msg=msg & toQuotedPrintable(private.msgParts[i].content)>
			</cfif>
			<cfset msg=msg & CRLF>
			
		</cfloop>	
		
		
		<!---
					Indicate end of message
					--->
		<cfset msg=msg & "--" & boundary & "--" & CRLF & CRLF>
		
		
		<!---
					Add footer
					--->
		<cfset msg=msg & private.footer & CRLF>
		
		
		<cfreturn msg>
								
	</cffunction>


	<!---
				toQuotedPrintable
				
				Utility function to encode a string as quoted-printable
				according to rules in RFC 2045.
				--->
	<cffunction
		name="toQuotedPrintable"
		returnType="string"
		access="public"
		output="false">
		
		<cfargument
			name="in"
			type="string"
			required="true" />
		
		<cfset var inCounter=0>
		<cfset var lineCounter=1>
		<cfset var thisChar="">
		<cfset var thisCharAsc=0>
		<cfset var out="">
		<cfset var prevWasCr="no">
      
		<cfif Len(arguments.in)>
			
			<cfloop index="inCounter" from="1" to="#Len(arguments.in)#">

				<cfset thisChar=mid(arguments.in,inCounter,1)>
				<cfset thisCharAsc=asc(thisChar)>
				
				
				<!---
							If it's a CRLF, output as is and
							reset line counter.
							--->
				<cfif thisCharAsc IS "13">
					<cfset out=out & CRLF>
					<cfset inCounter=inCounter+1>
					<cfset lineCounter=1>			
				   <cfset prevWasCr="yes">
				
				<cfelseif thisCharAsc IS "10">
               
               <cfif Not prevWasCr>
   					<cfset out=out & CRLF>
   					<cfset lineCounter=1>			
               </cfif>
				   <cfset prevWasCr="no">
               


				<!---
							If this is US-ASCII char, then copy as is.
							--->
				<cfelseif (thisCharAsc GTE 32 AND thisCharAsc LTE 60) OR
									(thisCharAsc GTE 62 AND thisCharAsc LTE 126)>
				   <cfset prevWasCr="no">
					<cfset out=out & thisChar>
					<cfset lineCounter=lineCounter+1>
				
				
				<!---
							Otherwise, encode the character.
							--->
				<cfelse>
					
				   <cfset prevWasCr="no">
					<cfset thisChar=UCase(FormatBaseN(thisCharAsc,16))>
					<cfif Len(thisChar) IS 1>
						<cfset thisChar="0" & thisChar>
					</cfif>
					<cfset thisChar="=" & thisChar>
					
					<cfif lineCounter GT 72>
						<cfset out=out & "=" & CRLF>
						<cfset lineCounter=1>
					</cfif>
					
					<cfset out=out & thisChar>
					<cfset lineCounter=lineCounter+3>
					
				</cfif>
				

				<!---
							Maximum line length is 75 chars, not including
							the soft line-break character.
							--->
				<cfif lineCounter IS "75">
					<cfset out=out & "=" & CRLF>
					<cfset lineCounter=1>
				</cfif>
				
			</cfloop>
		</cfif>	
		
		<cfreturn out>
		
	</cffunction>
	
	
	<!---
				toBase64Short
				
				Converts binary input to Base64 and trims
				each line to 76 characters.
				--->
	<cffunction
		name="toBase64Short"
		returnType="string"
		access="public"
		output="false">
		
		<cfargument
			name="in"
			type="binary"
			required="true" />
			
		<cfset var b64=toBase64(arguments.in)>
		<cfset var out="">	
		<cfset var i=0>
		
		<cfloop index="i" from="1" to="#Len(b64)#" step="76">
			<cfset out=out & Mid(b64,i,76) & CRLF>
		</cfloop>
		
		<cfreturn out>
		
	</cffunction>				
</cfcomponent>			